capture program drop fitstat
*! version 1.0.0 9/9/96
* -fitstat- measures of fit for ml models
* do:  error if not following valid S_E_cmd
* do:  Seems to need to run right after logit or logistic, not after lstat etc.
* do:  add poisson, nbreg

program define fitstat
    version 4.0
    tempvar n ll1 df x2 r2 ll0 b varys v vary vx bnocon vare
    tempvar twon factor r2mcf r2mcfa r2mz r2ml r2cu isbin
    tempvar isord isvare islgt ssr r2 r2adj aic bic bicp
    tempvar nparm dev lrx2
    mat `vare' = J(1,1,1)

* get values from _result
    if "$S_E_cmd"~="regress" {
        sca `n' = _result(1)
        sca `ll1' = _result(2)
        sca `df' = _result(3)
        sca `x2' = _result(6)
        sca `r2' = _result(7)
        sca `ll0' = ((`x2'/2)-`ll1')*-1
    }

    if "$S_E_cmd"=="regress" {
        sca `n' = _result(1)
        sca `ssr' = _result(4)
        sca `r2' = _result(7)
        sca `r2adj' = _result(8)
        sca `ll1' = -(`n'/2) * (1 + ln(2*_pi) + ln(`ssr'/`n'))
        mat `vare'[1,1] = (_result(9)*_result(9)*_result(5))/_result(1)
        quietly sum $S_E_depv
        sca `vary' = _result(4)
        sca `ll0' = -(`n'/2)* (1 + ln(2*_pi) + ln(`vary'*(`n'-1)/`n'))
        sca `varys' = `vary'
        sca `vary' = _result(4)*(`n'-1)/(`n')
    }

* get some basic information
    local isbin = "no"
    if "$S_E_cmd"=="probit" | "$S_E_cmd"=="logit" /*
    */ | "$S_E_cmd"=="logistic" {
        local isbin = "yes";
    }
    local isord = "no"
    if "$S_E_cmd"=="oprobit" | "$S_E_cmd"=="ologit" {
        local isord = "yes";
    }
    local isvare = "no"
    if "$S_E_cmd"=="probit" | "$S_E_cmd"=="logit" /*
    */ | "$S_E_cmd"=="logistic" /*
    */ | "$S_E_cmd"=="oprobit" | "$S_E_cmd"=="ologit" {
        local isvare = "yes";
    }
    local islgt = "no"
    if "$S_E_cmd"=="logit" | "$S_E_cmd"=="ologit" /*
    */ | "$S_E_cmd"=="logistic" {
        local islgt = "yes";
    }

    mat `b' = get(_b)
    local nvars = colsof(`b') - 1
    sca `nparm' = colsof(`b')

    if "`isbin'"=="yes" {
        local ncats = 2
    }
    if "$S_E_cmd"=="mlogit" {
        sca `nparm' = colsof(`b') * rowsof(`b')
        local ncats = rowsof(`b') + 1
    }

* determine # of categories if ordered regression model
    if "`isord'"=="yes" {
        local ncats=1
        capture local j=_b[_cut`ncats']
        while _rc==0 {
            local ncats=`ncats'+1
            capture local j=_b[_cut`ncats']
        }
        local nvars = `nvars' - `ncats' + 1 /* #x + 1 for lhs var */
    }

 * strip off names of _con and _cut
	local varlist : colnames(`b')
	parse "`varlist'", parse (" ")
    if "`isord'"=="yes" {
	    local locat = index("`varlist'","_cut")-2
	    local rhsnam = substr("`varlist'",1,`locat')
    }
    if "`isbin'"=="yes" | "$S_E_cmd"=="regress" {
    	local locat = index("`varlist'","_cons")-2
	    local rhsnam = substr("`varlist'",1,`locat')
    }

* header
    di " "
    di "Measures of Fit for " in w "$S_E_cmd" in y " on " in w "$S_E_depv"
    di " "

* get cov matrix of observed variables

    if "`isvare'"=="yes" {
        sca `factor' = 1/(`n'-1)  /* 1 over nobs - 1 */
        quietly mat accum `vx' = $S_E_depv `rhsnam', deviations noconstant
        sca `vary' = `factor' * `vx'[1,1]
        mat `vx' = `vx'[2...,2...] /* trim off lhs variable */
        mat `vx' = `factor' * `vx' /* cov matrix of rhs vars */
        mat `bnocon' = `b'[1,1..`nvars'] /* trim off _con */
        mat `v' = `bnocon' * `vx'
        mat `v' = `v' * `bnocon''
        /* probit assumes variance of 1 */
        scalar `factor' = 1
        if "`islgt'"=="yes" {
            scalar `factor' = (_pi*_pi)/3
        }
        mat `vare' = `factor' * `vare'
        mat `v' = `v' + `vare'
        sca `varys' = `v'[1,1]
    }

    local c1 = 27
    local c2 = 40
    local c3 = 65

    di "Log-Lik Intercept Only:" _col(`c1') %9.4f `ll0' /*
    */ _col(`c2') "Log-Lik Full Model:" _col(`c3') %9.4f `ll1'

    sca `r2mcf' = 1 - (`ll1'/`ll0')
    sca `r2mcfa' = 1 - ((`ll1'-`nvars')/`ll0')
    di "McFadden's R2:" _col(`c1') %9.4f `r2mcf' /*
    */ _col(`c2') "McFadden's Adj R2:" _col(`c3') %9.4f `r2mcfa'

    sca `twon' = 2/`n'
    sca `r2ml' = 1 - ((exp(`ll0')/exp(`ll1'))^`twon')
    sca `r2cu' = (`r2ml')/(1-(exp(`ll0'))^`twon')
    di "Max Lik R2:" _col(`c1') %9.4f `r2ml'  /*
    */ _col(`c2') "Cragg & Uhler's R2:" _col(`c3') %9.4f `r2cu'

    if "`isvare'"=="yes" | "$S_E_cmd"=="regress" {
        di "Variance of y*:" _col(`c1') %9.0g `varys' /*
        */ _col(`c2') "Variance of error:" _col(`c3') %9.0g `vare'[1,1]
        sca `r2mz' = (`varys' - `vare'[1,1])/`varys'
        di "McKelvey and Zavoina's R2:" _col(`c1') %9.4f `r2mz'
    }

    if "$S_E_cmd"=="regress" {
        di "R2:" _col(`c1') %9.4f `r2' /*
        */ _col(`c2') "Adjusted R2:" _col(`c3')  %9.4f `r2adj'
    }

    sca `aic' = ((-2*`ll1') + (2*`nparm'))/`n'
    sca `dev' = -2*`ll1'
    sca `lrx2' = 2*(`ll1'-`ll0')
    local dfd = `n'-`nparm'
    local dflrx2 = `nvars'
    sca `bic' = `dev' - (`dfd'*ln(`n'))
    sca `bicp' = -`lrx2' + (`dflrx2'*ln(`n'))
    di "D(" `dfd' "):   " _col(`c1') %9.3f `dev' /*
    */ _col(`c2')  "G2(" `dflrx2' "):"  _col(`c3') %9.4g `lrx2'
    di "AIC:" %9.4g _col(`c1') `aic'
    di "BIC:"  _col(`c1') %9.4g `bic' /*
    */ _col(`c2') "BIC':"  _col(`c3') _col(60) %9.4g `bicp'

    end

