capture program drop genpred
*! version 1.1.2 6/17/97 - new syntax
*! version 1.1.1 6/17/97 - store and preserve
*! version 1.1.0 6/16/97
*! version 1.0.9 4/7/97 - add root name
* version 1.0.8 4/7/97 - poisson & nbreg
* version 1.0.7 9/6/96
*
* -genpred- generate a data set for plotting predicted values.

program define genpred
    version 5.0
    tempvar b alpha ai gai wasrun
    
    if (    "$S_E_cmd"!="poisson"  & "$S_E_cmd"!="nbreg" /*
       */ & "$S_E_cmd"!="logit"  & "$S_E_cmd"!="probit" /*
       */ & "$S_E_cmd"!="ologit" & "$S_E_cmd"!="oprobit" /*
       */ & "$S_E_cmd"!="mlogit") {
        di
        di in r "-genpred- does not work with the last estimation command used."
        exit
    }

* make sure base value is defined
    capture local tmp = jsl_base[1,1]
    if _rc != 0 {
        di " "
        di in r "You must use -tobase- to set the base values."
        exit
    }

    preserve
    quietly catnm $S_E_depv
    global jsl_catv "$S_3"
    global jsl_catn "$S_1"        
    
* get names of independent variables
    mat `b' = get(_b)
    if "$S_E_cmd"=="nbreg" {
        local nb = colsof(`b')
        scalar `alpha' = exp(`b'[1,`nb'])
        scalar `ai' = 1/`alpha'
        scalar `gai' = exp(lngamma(`ai'))    
        local nb = colsof(`b') - 1 /* -1 for alpha */        
        mat `b' = `b'[1,1..`nb']
    }

* determine # of categories if ordered regression model
    if "$S_E_cmd"=="oprobit" | "$S_E_cmd"=="ologit" {
        local ncats 1
        capture local j=_b[_cut`ncats']
        while _rc==0 {
            local ncats=`ncats'+1
            capture local j=_b[_cut`ncats']
        }
    }
    else if "$S_E_cmd"=="mlogit" {
        local ncats = rowsof(`b') + 1
    }

* parse input
	local varlist "req ex max(1)"
	local options "From(real 0) To(real 100) Ncases(integer 100) GENerate(string)"
	parse "`*'"
	if "`from'" == ""|"`to'" == "" | "`generate'" == "" {
	    di
	    di in r "You must specify the options From(), To() and Generate()."
	    di in r "Syntax is: genpred chngvar,From(#) To(#) Generate(label)"
	    exit
    }	    
    if `from'>=`to' {
        di
        di in r "The from value must be less than the to value."
        exit
     }
    local root = substr("`generate'",1,5) /* max length of 5 */
    local prednm = "`root'P"    
    local chngvar = "`varlist'"
    local varlist : colnames(`b')
    drop _all
    quietly set obs `ncases'
    generate _genpred = 1 /* marker variable to show genpred was run */

* add new variables equal to values in jsl_base    
    local nrhs = colsof(jsl_base) - 1
    getrhs
    local nrhs2 = $S_2
    if "`nrhs2'"~="`nrhs'" {
        di
        di in r "jsl_base has the wrong number of values:"
        tobase
        exit
    }
    local i = 1
    local todrop ""
    local chngok "no"
    while `i'<=`nrhs' {
        local varnm : word `i' of `varlist'
        if "`chngvar'" == "`varnm'" { local chngok "yes" }
        local i=`i'+1 
        local todrop `todrop' `varnm'
        capture drop `varnm'
        gen `varnm' = jsl_base[1,`i']
    }

    if "`chngok'" == "no" {
        di
        di in r "Variable " in y "`chngvar'" /*
            */ in r " is not an independent variable."
        exit
    }

* in data set make chngvar vary from from to to
    drop `chngvar'  /* drop current values of change variable */
    range `chngvar' `from' `to'
    gen `root'x = `chngvar' /* this variable is permanent */
    label variable `root'x "`chngvar'"

* if ordered, need set of names
    if "$S_E_cmd"=="oprobit" | "$S_E_cmd"=="ologit" {
        local prednm = ""
        local i = 1
        while `i'<=`ncats' {
            local va : word `i' of $jsl_catv
            local prednm `prednm' `root'P`va'
            local i=`i'+1 
        }
    }

* get predicted values
    if "$S_E_cmd"=="probit" | "$S_E_cmd"=="logit" {
        predict `prednm' /* add predicted values to data set */
        local nm : word 2 of $jsl_catn
        label variable `prednm' "P(`nm')"        
    }
    if "$S_E_cmd"=="oprobit" {
        oprobitp `prednm'
    }
    if "$S_E_cmd"=="ologit" {
        ologitp `prednm'
    }

    if "$S_E_cmd"=="mlogit" {
        local i = 1
        while `i'<=`ncats' {
            local va : word `i' of $jsl_catv
            predict `root'P`va',o(`va')
            local i = `i' + 1
        }
    }
    * create labels and cumulative probabilities
    if "$S_E_cmd"=="oprobit" | "$S_E_cmd"=="ologit" | "$S_E_cmd"=="mlogit"{
        local i = 1
        while `i'<=`ncats' {
            local nm : word `i' of $jsl_catn
            local va : word `i' of $jsl_catv
            label variable `root'P`va' "Pr(`nm')"
            local i = `i'+1
        }
        local nm : word 1 of $jsl_catn
        local va : word 1 of $jsl_catv
        local lbl `va'
        gen `root'S`va' = `root'P`va'
        label variable `root'S`va' "Pr(`lbl')"
        local vapre `va'
        local i = 2
        while `i'<=`ncats' {
            local nm : word `i' of $jsl_catn
            local va : word `i' of $jsl_catv
            gen `root'S`va' = `root'S`vapre' + `root'P`va'
            local vapre `va'
            local lbl "`lbl'|`va'"
            label variable `root'S`va' "Pr(`lbl')"
            local i = `i'+1
        }
    }
    if "$S_E_cmd"=="poisson" {
        local munm = "`root'Mu"    
        predict `munm'
    	replace `munm' = exp(`munm')
        local prednm = "`root'P"        	
	    local i = 0
	    while `i' <= 9 {
        	local i1 = `i' + 1
            gen `prednm'`i' = exp(-`munm')*(`munm'^`i')/   /*
            */ round(exp(lnfact(`i')),1)
	        local i=`i'+1
	    }
    }
    if "$S_E_cmd"=="nbreg" {
        local munm = "`root'Mu"        
        predict `munm'
    	replace `munm' = exp(`munm')
        local prednm = "`root'P"        	    	
	    local i = 0
	    while `i' <= 9 {
        	local i1 = `i' + 1
            gen `prednm'`i' = (  exp(lngamma(`i'+`ai')) /*
            */ / ( round(exp(lnfact(`i')),1) * exp(lngamma(`ai')) )  ) /*
            */ * ((`ai'/(`ai'+`munm'))^`ai') * ((`munm'/(`ai'+`munm'))^`i')
	        local i=`i'+1
	    }
    }


    drop `todrop'
    capture save _tmp_,replace
    restore
    merge using _tmp_
    drop _merge

* tell us what you did
    di " "
    di in g "Predicted values as " in w "`chngvar'" in g /*
     */ " varies from " in w "`from'" in g " to " /*
     */ in w "`to'" in g ". Variables held at:"
    matrix rown jsl_base = Base=     
    mat list jsl_base,noheader f(%8.3f)

end

